//
//  CommonFunctions.m
//
//  Created by abhayam on 21/09/15.
//  Copyright (c) 2015 TTN. All rights reserved.
//

#import "CommonFunctions.h"

@interface NSObject ()
- (void)leftDrawerButtonPress:(id)sender;
- (void)backBtnAciton:(id)sender;
@end

#pragma mark - Notification

void removeNotification(id sender){
    [[NSNotificationCenter defaultCenter] removeObserver:sender];
}
void postNotification(NSString *name, id object){
    [[NSNotificationCenter defaultCenter] postNotificationName:name object:object];
}
void addNotification(NSString *name, id sender,SEL selector,id object){
    [[NSNotificationCenter defaultCenter] addObserver:sender selector:selector name:name object:object];
}

#pragma mark- Custom

void startShake (UIView* view)
{
    CGAffineTransform leftShake = CGAffineTransformMakeTranslation(-5, 0);
    CGAffineTransform rightShake = CGAffineTransformMakeTranslation(0, 0);
    
    view.transform = leftShake;  // starting point
    
    [UIView beginAnimations:@"shake_button" context:(__bridge void *)(view)];
    [UIView setAnimationRepeatAutoreverses:YES]; // important
    [UIView setAnimationRepeatCount:3];
    [UIView setAnimationDuration:0.06];
    
    shakeEnded(@"shake_button", YES, (__bridge void *)(view));
    view.transform = rightShake; // end here & auto-reverse
    [UIView commitAnimations];
}

void shakeEnded (NSString * animationID, BOOL finished, void *context)
{
    if (finished) {
        UIView* item = (__bridge UIView *)context;
        item.transform = CGAffineTransformIdentity;
    }
}

void showCustomAlert (NSString * title,NSString * message){
    UIAlertView *alerView = [[UIAlertView alloc]initWithTitle:title message:message delegate:nil cancelButtonTitle:nil otherButtonTitles:@"Ok", nil];
    [alerView show];
}

#pragma mark-  Custom Back Button

void customBackButtonForController (UIViewController *controller){
    
    UIButton *customBackBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    customBackBtn.frame = CGRectMake(0, 0, 22, 22);
    BUTTONIMAGE(customBackBtn, "backArrow");
    BUTTON_ADD_TARGET(customBackBtn,controller,backBtnAciton:);
    UIBarButtonItem *leftBarBtnItem = [[UIBarButtonItem alloc] initWithCustomView:customBackBtn];
    [controller.navigationItem setLeftBarButtonItem:leftBarBtnItem animated:YES];
}


CGSize dynamicHeightForLabel(NSString *text,float width, UIFont *font){
    NSLog(@"%@",text);
    
    CGSize constrainedSize = CGSizeMake(width, CGFLOAT_MAX);
    
    NSDictionary *attributesDictionary = [NSDictionary dictionaryWithObjectsAndKeys:font, NSFontAttributeName,nil];
    NSMutableAttributedString *string = [[NSMutableAttributedString alloc] initWithString:text attributes:attributesDictionary];
    CGRect requiredHeight = [string boundingRectWithSize:constrainedSize options:NSStringDrawingUsesLineFragmentOrigin context:nil];
    
    if (requiredHeight.size.width > width) {
        requiredHeight = CGRectMake(0, 0, width, requiredHeight.size.height);
    }
    
    return requiredHeight.size;
}



#pragma mark- Manage Drawer

void manageFlyOur(UIViewController *controller){
    
    UIButton *leftDrawerButton = [UIButton buttonWithType:UIButtonTypeCustom];
    leftDrawerButton.frame = CGRectMake(0, 0, 22, 22);
    BUTTONIMAGE(leftDrawerButton, "icon_menu");
    BUTTON_ADD_TARGET(leftDrawerButton, controller, leftDrawerButtonPress:);
    
    UIBarButtonItem *leftBarBtnItem = [[UIBarButtonItem alloc] initWithCustomView:leftDrawerButton];
    [controller.navigationItem setLeftBarButtonItem:leftBarBtnItem animated:YES];
}



#pragma mark- String
NSMutableAttributedString* attributedStringForString(NSString *string,NSString *fontName,CGFloat size, UIColor *color){
    string = isNonEmptyString(string)?string:@"";
    
    NSDictionary *attributesDictionary = [NSDictionary dictionaryWithObjectsAndKeys:[UIFont fontWithName:fontName size:size], NSFontAttributeName,color,NSForegroundColorAttributeName,nil];
    
    return [[NSMutableAttributedString alloc] initWithString:string attributes:attributesDictionary];
}


BOOL nonEmptyString(NSString *string){
    if (string == (id)[NSNull null] || string.length == 0){
        return NO;
    }else{
        NSMutableString *inputString = [NSMutableString stringWithString:string];
        [inputString replaceOccurrencesOfString:@" " withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [string length])];
        
        if(inputString.length == 0)
            return NO;
        return YES;
    }
}

BOOL nonEmptyDictionary(NSDictionary *dict){
    if (dict == (id)[NSNull null]){
        return NO;
    }else{
        return YES;
    }
}


#pragma mark - Date

NSDate* fixFormatDateFromString(NSString* timeString)
{
    NSDate *gmtDate = dateFromStringWithFormat(timeString, @"dd'.'MM'.'yyyy HH:mm:ss");
    NSDate* currentDate = timeWithGMTTime(gmtDate);
    return currentDate;
}

NSString* fixFormatDateStringFromString(NSString* timeString)
{
    NSDate* currentDate = fixFormatDateFromString(timeString);
    timeString = stringFromDate(currentDate, kFixDateTimeFormat);
    return timeString;
}


NSDate* timeWithGMTTime(NSDate* gmtTime)
{
    NSTimeZone *currentTimeZone = [NSTimeZone localTimeZone];
    NSTimeZone *utcTimeZone = [NSTimeZone timeZoneWithAbbreviation:@"UTC"];
    
    NSInteger currentGMTOffset = [currentTimeZone secondsFromGMTForDate:gmtTime];
    NSInteger gmtOffset = [utcTimeZone secondsFromGMTForDate:gmtTime];
    NSTimeInterval gmtInterval = currentGMTOffset - gmtOffset;
    NSDate *destinationDate = [[NSDate alloc] initWithTimeInterval:gmtInterval sinceDate:gmtTime];
    return destinationDate;
}

NSDate* exactTimeWith(NSDate* sourceDate)
{
    //get current time
    NSTimeZone* sourceTimeZone = [NSTimeZone timeZoneWithAbbreviation:@"GMT"];
    NSTimeZone* destinationTimeZone = [NSTimeZone systemTimeZone];
    NSInteger sourceGMTOffset = [sourceTimeZone secondsFromGMTForDate:sourceDate];
    NSInteger destinationGMTOffset = [destinationTimeZone secondsFromGMTForDate:sourceDate];
    NSTimeInterval interval = destinationGMTOffset - sourceGMTOffset;
    NSDate* currentTime = [[NSDate alloc] initWithTimeInterval:interval sinceDate:sourceDate];
    return currentTime;
}

NSDate* beginningOfDay(NSDate *date)
{
    NSString* string = stringFromDate(date, kDateFormat);
    date = dateFromStringWithFormat(string, kDateFormat);
    return date;
}

NSDate* endOfDay(NSDate *date)
{
    NSString* string = stringFromDate(date, kDateFormat);
    string = [string stringByAppendingString:@" 23:59:59"];
    date = dateFromStringWithFormat(string, @"yyyy-MM-dd HH:mm:ss");
    return date;
}


NSDate* dateFromInterval(NSTimeInterval interval)
{
    if(interval == 0)
        return nil;
    NSDate* date = [NSDate dateWithTimeIntervalSince1970:interval];
    return date;
}

NSDate* gmtDateWithDate(NSDate* date)
{
    NSTimeZone *currentTimeZone = [NSTimeZone localTimeZone];
    NSTimeZone *utcTimeZone = [NSTimeZone timeZoneWithAbbreviation:@"UTC"];
    
    NSInteger currentGMTOffset = [currentTimeZone secondsFromGMTForDate:date];
    NSInteger gmtOffset = [utcTimeZone secondsFromGMTForDate:date];
    NSTimeInterval gmtInterval = currentGMTOffset - gmtOffset;
    gmtInterval = -gmtInterval;
    NSDate *destinationDate = [[NSDate alloc] initWithTimeInterval:gmtInterval sinceDate:date];
    //DEBUG_LOG(@"gmtDateWithDate %@",destinationDate);
    return destinationDate;
}


NSDate* dateWithCurrentTimeZone(NSDate* date)
{
    NSTimeZone *currentTimeZone = [NSTimeZone localTimeZone];
    NSTimeZone *utcTimeZone = [NSTimeZone timeZoneWithAbbreviation:@"UTC"];
    
    NSInteger currentGMTOffset = [currentTimeZone secondsFromGMTForDate:date];
    NSInteger gmtOffset = [utcTimeZone secondsFromGMTForDate:date];
    NSTimeInterval gmtInterval = currentGMTOffset - gmtOffset;
    NSDate *destinationDate = [[NSDate alloc] initWithTimeInterval:gmtInterval sinceDate:date];
    //DEBUG_LOG(@"dateWithCurrentTimeZone %@",destinationDate);
    return destinationDate;
}

NSDate* dateFromStringWithFormat(NSString* str,NSString* format)
{
    if(!isNonEmptyString(str))
        str = @"";
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
    [dateFormatter setLocale:locale];
    [dateFormatter setDateFormat:format];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
    NSDate *date = [dateFormatter dateFromString:str];
    
    return date;
}

NSDate* dateFromString(NSString* str,NSString* format)
{
    if(!isNonEmptyString(str))
        return [NSDate new];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
    [dateFormatter setLocale:locale];
    [dateFormatter setDateFormat:format];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
    NSDate *date = [dateFormatter dateFromString:str];
    return date;
}

NSString* stringFromDate(NSDate* date,NSString* format)
{
    if(!date)
        return @"";
    if([format rangeOfString:kTimeFormat].location != NSNotFound)
    {
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *components = [calendar components:(NSHourCalendarUnit | NSMinuteCalendarUnit) fromDate:date];
        NSInteger minute = [components minute];
        if(minute == 0)
        {
            format = @"ha";
        }
    }
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
    [dateFormatter setLocale:locale];
    [dateFormatter setDateFormat:format];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
    
    return [dateFormatter stringFromDate:date];
}


#pragma mark- UIFont
UIFont* fontWithName(NSString* name,float size)
{
    return [UIFont fontWithName:name size:size];
}

#pragma mark - NSUserDefaults
void saveDataWithKey(id data,NSString *key)
{
    NSData *data_ = [NSKeyedArchiver archivedDataWithRootObject:data];
    [[NSUserDefaults standardUserDefaults] setObject:data_ forKey:key];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

id dataWithKey(NSString *key)
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:key];
    if(!data)
        return nil;
    return [NSKeyedUnarchiver unarchiveObjectWithData:data];
}


void removeAllCachedData (){
    USER_DEFAULT_SET_OBJECT(nil, kAccessToken);
    USER_DEFAULT_SET_OBJECT(nil, kUserID);
    USER_DEFAULT_SET_OBJECT(nil, kLoginStatus);
    [UserModel resetSharedInstance];
}


BOOL validationForInteger(id value){
    NSNumberFormatter *formatter = [NSNumberFormatter new];
    formatter.numberStyle = NSNumberFormatterDecimalStyle;
    
    NSNumber *number;
    
    if([value isKindOfClass:[NSString class]])
        number = [formatter numberFromString:value];
    else
        number = value;
    if(!number)
        return NO;
    else
        return YES;
    
}


id sortOriginalArray(id originalArray){
    [originalArray sortUsingSelector:@selector(compare:)];
    return originalArray;
}

id sortedNewArray(id originalArray){
    return [originalArray sortedArrayUsingSelector:@selector(compare:)];
}



#pragma mark - <Shadow Circle>
void addShadowCircle(CGContextRef context)
{
    CGSize  myShadowOffset = CGSizeMake (-10,  15);
    CGContextSaveGState(context);
    CGContextSetShadow (context, myShadowOffset, 5);
    CGContextSetLineWidth(context, 4.0);
    CGContextSetStrokeColorWithColor(context,
                                     [UIColor blueColor].CGColor);
    CGRect rectangle = CGRectMake(60,170,200,200);
    CGContextAddEllipseInRect(context, rectangle);
    CGContextStrokePath(context);
    CGContextRestoreGState(context);
}

#pragma mark - NSFileManager

NSString* fileNameFromURL(NSString* url)
{
    NSArray *arr = [url componentsSeparatedByString:@"/"];
    return [arr lastObject];
}

BOOL isFileExist(NSString* path)
{
    return  [[NSFileManager defaultManager] fileExistsAtPath:path];
}

void removePath(NSString* path)
{
    [[NSFileManager defaultManager] removeItemAtPath:path error:nil];
}

void createDirectory(NSString* path)
{
    [[NSFileManager defaultManager] createDirectoryAtPath:path withIntermediateDirectories:NO attributes:nil error:nil];
}

BOOL isDirectoryExist(NSString* path)
{
    BOOL is = NO;
    if([[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&is])
        return YES;
    return NO;
}
